/* --------------------------------------------------------------------------------------
 *  Copyright (C) 2008 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 *
 * --------------------------------------------------------------------------------------
 * 
 *  This demonstration is meant to show the features of the MAX1407 ADC and the LCD 
 *  module on the MAXQ2000 EVKIT. It implements a simple digital volt meter that will
 *  measure a DC voltage from 0 to +10 volts.
 *
 *
 *  Turn ON the following switches for this demonstration:
 *
 *       SW6.8 : Enable the LED panel
 *       SW3   : All 8 switches to allow MAXQ to interface to the MAX1407 DAS
 * --------------------------------------------------------------------------------------
 */
#include <iomaxq200x.h>
#include <intrinsics.h>
#include "maxq2000ev_lcd.h"
#include "maxq2000_gpio.h"
#include "maxq2000ev_1407.h"


/******************************************************************************
 *  Voltmeter project definitions
 */
#define ADC_MUX_IN0_AGND  0x61    // IN0 connects to + and AGND to - of ADC
#define ADC_CAL           0x89    // MODE = 1 and STA1 = 1 for calibration
#define ADC_START         0x09    // MODE = run, GAIN = 1/3, BUFP = on, Uni-polar
#define CONVERT_RATE      30      // conversion rate of 30Hz
#define DRDY              PORTPIN_1
#define MINUS_SIGN        0x40    // display a minus on LCD 7-segment digit
#define DP                0x80    // decimal point for LCD 7-segment digits
#define MAX_DC_VOLTS      1050    // about 10.50V displays a '-E-'

/******************************************************************************
 *  Voltmeter project prototypes
 */
void display_voltage(long int voltage_reading);




/******************************************************************************
 *  MAXQ 0-10V DC Voltmeter:
 */
void main()
{
  long int     volt_reading = 0;    // accumulator for conversion values
  int          conversion_counter;  // counter for 30 conversions
  int          i;                   // for delay loops

  // make sure MAX1407 CS (P54) pin is an output and capable of being controlled
  // by firmware
  gpio_setoutputpin(4, PORTPIN_4);
  
  // initialize the LCD module supplied with the MAXQ2000 EVKIT
  lcd_init();

  // pull MAX1407 RESET pin to reset the chip
  gpio_setoutputpin(6, PORTPIN_4);
  PO6 &= ~PORTPIN_4;
  for (i = 100; i != 0; i-- );
  PO6 |= PORTPIN_4;

  // Initialize the MAX1407 ADC located on the MAXQ2000 EVKIT main board. The
  // max1407_init function sets up the SPI interface and sends a command that
  // causes the MAX1407 to enter Run mode.
  max1407_init();

  // Configure MAX1407 internal MUX so IN0 connects to + and AGND to - of ADC
  // and do a calibration.
  max1407_writeregister(MAX1407REG_MUX, ADC_MUX_IN0_AGND, LENGTH_8);
  max1407_writeregister(MAX1407REG_ADC, ADC_CAL, LENGTH_8);

  // initiate continuous ADC conversions at 30Hz rate
  max1407_writeregister(MAX1407REG_ADC, ADC_START, LENGTH_8);
  max1407_writeregister(MAX1407REG_RUN, 0, LENGTH_8);

  // make sure MAX1407 DRDY (P61) pin is an input
  gpio_setinputpin(6, PORTPIN_1);
  conversion_counter = CONVERT_RATE;

  for(;;)
  {
    // sample the MAX1407 DRDY pin to see if an ADC conversion is available
    while ( PI6 & DRDY );

    // The sample is accumulated in the volt_reading variable. The MAX1407 was
    // set for a 30Hz sample rate. After 30 samples have been accumulated the
    // sum is divided by 30 and then displayed on the LCD. Therefore, the
    // LCD is updated once per second.
    volt_reading += (long int)(max1407_readregister(MAX1407REG_DATA, LENGTH_16));
    
    // accumulate thirty samples, then display the value on the LCD
    if (--conversion_counter == 0)
    {
      volt_reading /= CONVERT_RATE;    // calculate average

      // The 1.00K and 3.00K resistors divide the VIN by 4. So, VIN = 10V will
      // be 2.5V at IN0. The VGA is set for 1/3, so 3*1.25V = 3.75V = 0xFFFF.
      // Therefore, (2.5V/3.75V)*65536 = 43690.667. So, VIN = 0V will yield 0
      // and VIN = 10V will yield approximately 43690.
      //
      // The conversion calculation will be: (ADC data/43690)*1000.
      volt_reading = ((volt_reading * 1000) / 43690);

      // The volt_reading must now be displayed on the LCD. If volt_reading = 1 then it
      // will be displayed as '0.01.' If volt_reading = 1000 will be displayed as '10.00.'
      display_voltage(volt_reading);

      // finish up by reseting the variables for another CONVERT_RATE samples
      conversion_counter = CONVERT_RATE;
      volt_reading = 0;
    }
  }
}




/******************************************************************************
 *  This function will display the voltage reading on the LCD. The number
 *  passed (nominally 0 .. 1000) will be displayed as '0.00' .. '10.00.'
 *  An error is displayed '-E-' if the value passed is less than 0 or
 *  greater than MAX_DC_VOLTS.
 */
void display_voltage(long int voltage_reading)
{
  if (( voltage_reading <= MAX_DC_VOLTS ) && ( voltage_reading >= 0 ))
  {
    LCD4 = 0;                   // blank minus sign, colons, etc.
    LCD3 = 0;                   // blank LCD 1,000's digit
    
    if ( voltage_reading < 10 )
    {
      LCD2 = LCD_PATTERN_0;     // LCD 100's digit = '0'
      LCD1 = LCD_PATTERN_0+DP;  // LCD 10's digit = '0'
      LCD0 = lcd_getlcddigit((int) voltage_reading);
    }
    else if ( voltage_reading < 100 )
    {
      LCD2 = LCD_PATTERN_0;     // LCD 100's digit = '0'
      LCD1 = (lcd_getlcddigit((int)(voltage_reading/10)))+DP;
      LCD0 = lcd_getlcddigit((int)(voltage_reading%10));
    }
    // voltage reading is 1.00V or higher
    else
    {
      LCD0 = lcd_getlcddigit((int)(voltage_reading%10));
      voltage_reading /= 10;
      LCD1 = (lcd_getlcddigit((int)(voltage_reading%10)))+DP;
      voltage_reading /= 10;
      LCD2 = lcd_getlcddigit((int)(voltage_reading%10));
      voltage_reading /= 10;
      if ( voltage_reading != 0 )
      {
        LCD3 = lcd_getlcddigit((int) voltage_reading);
      }
    }
  }
  else
  {
    // Here if the voltage was passes as greater than MAX_DC_VOLTS.
    // Display '-E-'
    LCD4 = 0;                 // blank minus sign, colons, etc.
    LCD3 = 0;                 // blank LCD 1,000's digit
    LCD2 = MINUS_SIGN;        // display a minus on 100's digit
    LCD1 = LCD_PATTERN_E;     // display an 'E' on 10's digit
    LCD0 = MINUS_SIGN;        // display a minus on 1's digit
  }
}
