/* ----------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 *
 *  Module:        maxq2000_spi
 *  Description:   Utility functions for configuring and using the MAXQ2000's
 *                 SPI module.
 *  Filename:      maxq2000_spi.c
 *  Date:          November 2, 2004
 *  Version:       1.00
 *
 * ---------------------------------------------------------------------------*/

#include "maxq2000_spi.h"
#include <iomaxq200x.h>

void spi_enable(const unsigned char spien)
{
    PD5 |= 0x070;  /* Set SPI output pins (CS, SCLK, DOUT) as output. */
    PD5 &= ~0x080; /* Set SPI input pin (DIN) as input.               */
    SPICN_bit.SPIEN = (spien ? 1 : 0);
}

void spi_setmode(const spi_mode mstm)
{
    SPICN_bit.MSTM = (mstm ? 1 : 0);
}

void spi_setclockpolarity(const spi_polarity ckpol)
{
    SPICF_bit.CKPOL = (ckpol ? 1 : 0);
}

void spi_setclockphase(const spi_phase ckpha)
{
    SPICF_bit.CKPHA = (ckpha ? 1 : 0);
}

void spi_setcharacterlength(const spi_size chr)
{
    SPICF_bit.CHR = (chr ? 1 : 0);
}

unsigned long spi_setbaudrate(const unsigned long baudrate,
                              const unsigned long systemfreq)
{
    /* Compute the new value for SPICK           */
    /* baudrate = systemfreq / (2 * (SPICK + 1)) */
    unsigned long spick = ((systemfreq / baudrate) >> 1);

    /* Only divide ratios between 1 & 256 are supported. */
    spick &= 0x0FF;

    /* Set the SPICK register. */
    SPICK = (spick & 0x0FF);

    /* Return the actual value achieved. */
    return (systemfreq / ((spick + 1) << 1));
}

void spi_enablemodefaultdetection(const unsigned char modfe)
{
    SPICN_bit.MODFE = (modfe ? 1 : 0);
}

unsigned char spi_geterrors(const unsigned char clearerrors)
{
    unsigned char errors = SPICN & SPIFLAGS_ALLERRORS;
    if(clearerrors)
    {
    	SPICN &= ~SPIFLAGS_ALLERRORS;
    }
    return errors;
}

void spi_enableinterrupts(const unsigned char espii)
{
    SPICF_bit.ESPII = (espii ? 1 : 0);
}

unsigned char spi_getinterruptsources(const unsigned char clearinterrupts)
{
    unsigned char interrupts = SPICN & SPIFLAGS_ALL;
    if(clearinterrupts)
    {
    	SPICN &= ~SPIFLAGS_ALL;
    }
    return interrupts;
}

unsigned int spi_transmit(const unsigned int ch)
{
    unsigned int spib;
    
    while(SPICN_bit.STBY);      /* Wait until the character can be sent. */
    SPICN_bit.SPIC = 0;         /* In case it is already set, clear the  */
                                /*   transfer complete flag.             */
    SPIB = ch;                  /* Send the data.                        */
    while(!SPICN_bit.SPIC);     /* Wait for the transfer to complete.    */
    spib = SPIB;                /* Get the data received.                */
    SPICN_bit.SPIC = 0;         /* Clear the tansfer complete flag.      */
    return spib;
}

